@@ -12,18 +12,10 @@ $serv->set(array(
 		//'daemonize' => true,
 		//'log_file' => '/tmp/swoole.log'
 ));
-$serv->on('timer', function($serv, $interval) {
-	echo "onTimer: $interval\n";
-});
-
-$serv->on('start', function($serv) {
-	//$serv->addtimer(1000);
-});
 
 $serv->on('workerStart', function($serv, $worker_id) {
 	echo "{$worker_id} start".PHP_EOL;
 	$serv->workerid = $worker_id;
-	//if($worker_id == 0) $serv->addtimer(1000);
 });
 
 $serv->on('connect', function ($serv, $fd, $from_id){
@@ -57,4 +49,4 @@ $serv->on('close', function ($serv, $fd, $from_id) {
 	unset($serv->fdlist[$fd]);
 });
 
-$serv->start();
\ No newline at end of file
+$serv->start();
