@@ -18,6 +18,7 @@
 #include "Server.h"
 #include "Client.h"
 #include "socks5.h"
+#include "async.h"
 
 static int swClient_inet_addr(swClient *cli, char *host, int port);
 static int swClient_tcp_connect_sync(swClient *cli, char *host, int port, double _timeout, int udp_connect);
@@ -40,6 +41,8 @@ static int swClient_onStreamRead(swReactor *reactor, swEvent *event);
 static int swClient_onWrite(swReactor *reactor, swEvent *event);
 static int swClient_onError(swReactor *reactor, swEvent *event);
 static void swClient_onTimeout(swTimer *timer, swTimer_node *tnode);
+static void swClient_onResolveCompleted(swAio_event *event);
+static int swClient_onPackage(swConnection *conn, char *data, uint32_t length);
 
 static sw_inline void execute_onConnect(swClient *cli)
 {
@@ -100,7 +103,15 @@ int swClient_create(swClient *cli, int type, int async)
 
     if (async)
     {
-        cli->socket = swReactor_get(SwooleG.main_reactor, sockfd);
+        if (swIsMaster() && SwooleTG.type == SW_THREAD_REACTOR)
+        {
+            cli->reactor = SwooleTG.reactor;
+        }
+        else
+        {
+            cli->reactor = SwooleG.main_reactor;
+        }
+        cli->socket = swReactor_get(cli->reactor, sockfd);
     }
     else
     {
@@ -123,12 +134,12 @@ int swClient_create(swClient *cli, int type, int async)
     if (async)
     {
         swSetNonBlock(cli->socket->fd);
-        if (!swReactor_handle_isset(SwooleG.main_reactor, SW_FD_STREAM_CLIENT))
+        if (!swReactor_handle_isset(cli->reactor, SW_FD_STREAM_CLIENT))
         {
-            SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_STREAM_CLIENT | SW_EVENT_READ, swClient_onStreamRead);
-            SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_DGRAM_CLIENT | SW_EVENT_READ, swClient_onDgramRead);
-            SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_STREAM_CLIENT | SW_EVENT_WRITE, swClient_onWrite);
-            SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_STREAM_CLIENT | SW_EVENT_ERROR, swClient_onError);
+            cli->reactor->setHandle(cli->reactor, SW_FD_STREAM_CLIENT | SW_EVENT_READ, swClient_onStreamRead);
+            cli->reactor->setHandle(cli->reactor, SW_FD_DGRAM_CLIENT | SW_EVENT_READ, swClient_onDgramRead);
+            cli->reactor->setHandle(cli->reactor, SW_FD_STREAM_CLIENT | SW_EVENT_WRITE, swClient_onWrite);
+            cli->reactor->setHandle(cli->reactor, SW_FD_STREAM_CLIENT | SW_EVENT_ERROR, swClient_onError);
         }
     }
 
@@ -148,12 +159,14 @@ int swClient_create(swClient *cli, int type, int async)
             cli->send = swClient_tcp_send_sync;
             cli->sendfile = swClient_tcp_sendfile_sync;
         }
+        cli->reactor_fdtype = SW_FD_STREAM_CLIENT;
     }
     else
     {
         cli->connect = swClient_udp_connect;
         cli->recv = swClient_udp_recv;
         cli->send = swClient_udp_send;
+        cli->reactor_fdtype = SW_FD_DGRAM_CLIENT;
     }
 
     cli->_sock_domain = _domain;
@@ -167,6 +180,7 @@ int swClient_create(swClient *cli, int type, int async)
     cli->protocol.package_length_size = 4;
     cli->protocol.package_body_offset = 0;
     cli->protocol.package_max_length = SW_BUFFER_INPUT_SIZE;
+    cli->protocol.onPackage = swClient_onPackage;
 
     return SW_OK;
 }
@@ -176,11 +190,11 @@ int swClient_sleep(swClient *cli)
     int ret;
     if (cli->socket->events & SW_EVENT_WRITE)
     {
-        ret = SwooleG.main_reactor->set(SwooleG.main_reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_WRITE);
+        ret = cli->reactor->set(cli->reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_WRITE);
     }
     else
     {
-        ret = SwooleG.main_reactor->del(SwooleG.main_reactor, cli->socket->fd);
+        ret = cli->reactor->del(cli->reactor, cli->socket->fd);
     }
     if (ret)
     {
@@ -194,11 +208,11 @@ int swClient_wakeup(swClient *cli)
     int ret;
     if (cli->socket->events & SW_EVENT_WRITE)
     {
-        ret = SwooleG.main_reactor->set(SwooleG.main_reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_READ | SW_EVENT_WRITE);
+        ret = cli->reactor->set(cli->reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_READ | SW_EVENT_WRITE);
     }
     else
     {
-        ret = SwooleG.main_reactor->add(SwooleG.main_reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_READ);
+        ret = cli->reactor->add(cli->reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_READ);
     }
     if (ret)
     {
@@ -207,6 +221,54 @@ int swClient_wakeup(swClient *cli)
     return ret;
 }
 
+int swClient_shutdown(swClient *cli, int __how)
+{
+    if (!cli->socket || cli->socket->closed)
+    {
+        return SW_ERR;
+    }
+    if (__how == SHUT_RD)
+    {
+        if (cli->shutdown_read || cli->shutdow_rw || shutdown(cli->socket->fd, SHUT_RD))
+        {
+            return SW_ERR;
+        }
+        else
+        {
+            cli->shutdown_read = 1;
+            return SW_OK;
+        }
+    }
+    else if (__how == SHUT_WR)
+    {
+        if (cli->shutdown_write || cli->shutdow_rw || shutdown(cli->socket->fd, SHUT_RD) < 0)
+        {
+            return SW_ERR;
+        }
+        else
+        {
+            cli->shutdown_write = 1;
+            return SW_OK;
+        }
+    }
+    else if (__how == SHUT_RDWR)
+    {
+        if (cli->shutdow_rw || shutdown(cli->socket->fd, SHUT_RDWR) < 0)
+        {
+            return SW_ERR;
+        }
+        else
+        {
+            cli->shutdown_read = 1;
+            return SW_OK;
+        }
+    }
+    else
+    {
+        return SW_ERR;
+    }
+}
+
 #ifdef SW_USE_OPENSSL
 int swClient_enable_ssl_encrypt(swClient *cli)
 {
@@ -215,6 +277,15 @@ int swClient_enable_ssl_encrypt(swClient *cli)
     {
         return SW_ERR;
     }
+
+    if (cli->ssl_option.verify_peer)
+    {
+        if (swSSL_set_capath(&cli->ssl_option, cli->ssl_context) < 0)
+        {
+            return SW_ERR;
+        }
+    }
+
     cli->socket->ssl_send = 1;
 #if defined(SW_USE_HTTP2) && defined(SW_USE_OPENSSL) && OPENSSL_VERSION_NUMBER >= 0x10002000L
     if (cli->http2)
@@ -247,8 +318,29 @@ int swClient_ssl_handshake(swClient *cli)
     {
         return SW_ERR;
     }
+    if (cli->socket->ssl_state == SW_SSL_STATE_READY && cli->ssl_option.verify_peer)
+    {
+        if (swClient_ssl_verify(cli, cli->ssl_option.allow_self_signed) < 0)
+        {
+            return SW_ERR;
+        }
+    }
     return SW_OK;
 }
+
+int swClient_ssl_verify(swClient *cli, int allow_self_signed)
+{
+    if (swSSL_verify(cli->socket, allow_self_signed) < 0)
+    {
+        return SW_ERR;
+    }
+    if (cli->ssl_option.tls_host_name && swSSL_check_host(cli->socket, cli->ssl_option.tls_host_name) < 0)
+    {
+        return SW_ERR;
+    }
+    return SW_OK;
+}
+
 #endif
 
 static int swClient_inet_addr(swClient *cli, char *host, int port)
@@ -274,6 +366,9 @@ static int swClient_inet_addr(swClient *cli, char *host, int port)
         port = cli->http_proxy->proxy_port;
     }
 
+    cli->server_host = host;
+    cli->server_port = port;
+
     void *s_addr = NULL;
     if (cli->type == SW_SOCK_TCP || cli->type == SW_SOCK_UDP)
     {
@@ -311,13 +406,16 @@ static int swClient_inet_addr(swClient *cli, char *host, int port)
     {
         return SW_ERR;
     }
-    if (cli->async)
+    if (!cli->async)
     {
-        swWarn("DNS lookup will block the process. Please use swoole_async_dns_lookup.");
+        if (swoole_gethostbyname(cli->_sock_domain, host, s_addr) < 0)
+        {
+            return SW_ERR;
+        }
     }
-    if (swoole_gethostbyname(cli->_sock_domain, host, s_addr) < 0)
+    else
     {
-        return SW_ERR;
+        cli->wait_dns = 1;
     }
     return SW_OK;
 }
@@ -381,6 +479,14 @@ static int swClient_close(swClient *cli)
             sw_free(cli->ssl_option.tls_host_name);
         }
 #endif
+        if (cli->ssl_option.cafile)
+        {
+            sw_free(cli->ssl_option.cafile);
+        }
+        if (cli->ssl_option.capath)
+        {
+            sw_free(cli->ssl_option.capath);
+        }
     }
 #endif
     //clear buffer
@@ -401,9 +507,9 @@ static int swClient_close(swClient *cli)
     if (cli->async)
     {
         //remove from reactor
-        if (!cli->socket->removed && SwooleG.main_reactor)
+        if (!cli->socket->removed && cli->reactor)
         {
-            SwooleG.main_reactor->del(SwooleG.main_reactor, fd);
+            cli->reactor->del(cli->reactor, fd);
         }
         if (cli->timer)
         {
@@ -520,13 +626,17 @@ static int swClient_tcp_connect_sync(swClient *cli, char *host, int port, double
 static int swClient_tcp_connect_async(swClient *cli, char *host, int port, double timeout, int nonblock)
 {
     int ret;
+
     cli->timeout = timeout;
 
-    //alloc input memory buffer
-    cli->buffer = swString_new(cli->buffer_input_size);
     if (!cli->buffer)
     {
-        return SW_ERR;
+        //alloc input memory buffer
+        cli->buffer = swString_new(cli->buffer_input_size);
+        if (!cli->buffer)
+        {
+            return SW_ERR;
+        }
     }
 
     if (!(cli->onConnect && cli->onError && cli->onClose))
@@ -545,6 +655,56 @@ static int swClient_tcp_connect_async(swClient *cli, char *host, int port, doubl
         return SW_ERR;
     }
 
+    if (cli->wait_dns)
+    {
+        if (SwooleAIO.mode == SW_AIO_LINUX)
+        {
+            SwooleAIO.mode = SW_AIO_BASE;
+            SwooleAIO.init = 0;
+        }
+        if (SwooleAIO.init == 0)
+        {
+            swAio_init();
+        }
+
+        swAio_event ev;
+        bzero(&ev, sizeof(swAio_event));
+
+        int len = strlen(cli->server_host);
+        if (strlen(cli->server_host) < SW_IP_MAX_LENGTH)
+        {
+            ev.nbytes = SW_IP_MAX_LENGTH;
+        }
+        else
+        {
+            ev.nbytes = len + 1;
+        }
+
+        ev.buf = sw_malloc(ev.nbytes);
+        if (!ev.buf)
+        {
+            swWarn("malloc failed.");
+            return SW_ERR;
+        }
+
+        memcpy(ev.buf, cli->server_host, len);
+        ((char *) ev.buf)[len] = 0;
+        ev.flags = cli->_sock_domain;
+        ev.type = SW_AIO_DNS_LOOKUP;
+        ev.object = cli;
+        ev.callback = swClient_onResolveCompleted;
+
+        if (swAio_dispatch(&ev) < 0)
+        {
+            sw_free(ev.buf);
+            return SW_ERR;
+        }
+        else
+        {
+            return SW_OK;
+        }
+    }
+
     while (1)
     {
         ret = connect(cli->socket->fd, (struct sockaddr *) &cli->server_addr.addr, cli->server_addr.len);
@@ -561,7 +721,7 @@ static int swClient_tcp_connect_async(swClient *cli, char *host, int port, doubl
 
     if ((ret < 0 && errno == EINPROGRESS) || ret == 0)
     {
-        if (SwooleG.main_reactor->add(SwooleG.main_reactor, cli->socket->fd, cli->reactor_fdtype | SW_EVENT_WRITE) < 0)
+        if (cli->reactor->add(cli->reactor, cli->socket->fd, cli->reactor_fdtype | SW_EVENT_WRITE) < 0)
         {
             return SW_ERR;
         }
@@ -617,7 +777,7 @@ static int swClient_tcp_pipe(swClient *cli, int write_fd, int flags)
 static int swClient_tcp_send_async(swClient *cli, char *data, int length, int flags)
 {
     int n = length;
-    if (SwooleG.main_reactor->write(SwooleG.main_reactor, cli->socket->fd, data, length) < 0)
+    if (cli->reactor->write(cli->reactor, cli->socket->fd, data, length) < 0)
     {
         if (SwooleG.error == SW_ERROR_OUTPUT_BUFFER_OVERFLOW)
         {
@@ -692,12 +852,12 @@ static int swClient_tcp_sendfile_async(swClient *cli, char *filename, off_t offs
     {
         if (cli->socket->events & SW_EVENT_READ)
         {
-            return SwooleG.main_reactor->set(SwooleG.main_reactor, cli->socket->fd,
+            return cli->reactor->set(cli->reactor, cli->socket->fd,
                     cli->socket->fdtype | SW_EVENT_READ | SW_EVENT_WRITE);
         }
         else
         {
-            return SwooleG.main_reactor->add(SwooleG.main_reactor, cli->socket->fd,
+            return cli->reactor->add(cli->reactor, cli->socket->fd,
                     cli->socket->fdtype | SW_EVENT_WRITE);
         }
     }
@@ -720,7 +880,18 @@ static int swClient_tcp_recv_no_buffer(swClient *cli, char *data, int len, int f
         }
         if (errno == EINTR)
         {
-            continue;
+            if (cli->interrupt_time <= 0)
+            {
+                cli->interrupt_time = swoole_microtime();
+            }
+            else if (swoole_microtime() > cli->interrupt_time + cli->timeout)
+            {
+                break;
+            }
+            else
+            {
+                continue;
+            }
         }
 #ifdef SW_USE_OPENSSL
         if (errno == EAGAIN && cli->socket->ssl)
@@ -786,7 +957,7 @@ static int swClient_udp_connect(swClient *cli, char *host, int port, double time
 
         if (cli->async && cli->onConnect)
         {
-            if (SwooleG.main_reactor->add(SwooleG.main_reactor, cli->socket->fd, cli->reactor_fdtype | SW_EVENT_READ) < 0)
+            if (cli->reactor->add(cli->reactor, cli->socket->fd, cli->reactor_fdtype | SW_EVENT_READ) < 0)
             {
                 return SW_ERR;
             }
@@ -897,6 +1068,13 @@ static int swClient_https_proxy_handshake(swClient *cli)
 }
 #endif
 
+static int swClient_onPackage(swConnection *conn, char *data, uint32_t length)
+{
+    swClient *cli = (swClient *) conn->object;
+    cli->onReceive(conn->object, data, length);
+    return SW_OK;
+}
+
 static int swClient_onStreamRead(swReactor *reactor, swEvent *event)
 {
     int n;
@@ -943,7 +1121,7 @@ static int swClient_onStreamRead(swReactor *reactor, swEvent *event)
                 {
                     cli->socket->ssl_state = SW_SSL_STATE_WAIT_STREAM;
                 }
-                return SwooleG.main_reactor->set(SwooleG.main_reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_WRITE);
+                return cli->reactor->set(cli->reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_WRITE);
             }
             if (cli->onConnect)
             {
@@ -991,7 +1169,7 @@ static int swClient_onStreamRead(swReactor *reactor, swEvent *event)
                 {
                     cli->socket->ssl_state = SW_SSL_STATE_WAIT_STREAM;
                 }
-                return SwooleG.main_reactor->set(SwooleG.main_reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_WRITE);
+                return cli->reactor->set(cli->reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_WRITE);
             }
         }
         else
@@ -1041,7 +1219,7 @@ static int swClient_onStreamRead(swReactor *reactor, swEvent *event)
         }
         if (cli->_redirect_to_socket)
         {
-            ret = SwooleG.main_reactor->write(SwooleG.main_reactor, cli->_redirect_to_socket, buf, n);
+            ret = cli->reactor->write(cli->reactor, cli->_redirect_to_socket, buf, n);
         }
         else if (cli->_redirect_to_session)
         {
@@ -1189,6 +1367,28 @@ static void swClient_onTimeout(swTimer *timer, swTimer_node *tnode)
     }
 }
 
+static void swClient_onResolveCompleted(swAio_event *event)
+{
+    swClient *cli = event->object;
+    cli->wait_dns = 0;
+
+    if (event->error == 0)
+    {
+        swClient_tcp_connect_async(cli, event->buf, cli->server_port, cli->timeout, 1);
+    }
+    else
+    {
+        SwooleG.error = SW_ERROR_DNSLOOKUP_RESOLVE_FAILED;
+        cli->socket->removed = 1;
+        cli->close(cli);
+        if (cli->onError)
+        {
+            cli->onError(cli);
+        }
+    }
+    sw_free(event->buf);
+}
+
 static int swClient_onWrite(swReactor *reactor, swEvent *event)
 {
     swClient *cli = event->socket->object;
@@ -1211,13 +1411,13 @@ static int swClient_onWrite(swReactor *reactor, swEvent *event)
             {
                 if (_socket->ssl_want_read)
                 {
-                    SwooleG.main_reactor->set(SwooleG.main_reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_READ);
+                    cli->reactor->set(cli->reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_READ);
                 }
                 return SW_OK;
             }
         }
 #endif
-        if (swReactor_onWrite(SwooleG.main_reactor, event) < 0)
+        if (swReactor_onWrite(cli->reactor, event) < 0)
         {
             return SW_ERR;
         }
@@ -1240,7 +1440,7 @@ static int swClient_onWrite(swReactor *reactor, swEvent *event)
     if (SwooleG.error == 0)
     {
         //listen read event
-        SwooleG.main_reactor->set(SwooleG.main_reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_READ);
+        cli->reactor->set(cli->reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_READ);
         //connected
         _socket->active = 1;
         //socks5 proxy
