@@ -239,8 +239,11 @@ static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker)
     SwooleG.pid = getpid();
 
     SwooleWG.id = worker->id;
+    if (serv->max_request > 0)
+    {
+        SwooleWG.run_always = 0;
+    }
     SwooleWG.max_request = serv->max_request;
-    SwooleWG.request_count = 0;
     SwooleWG.worker = worker;
 
     SwooleTG.id = 0;
@@ -328,7 +331,7 @@ static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker)
     //close
     reactor->setHandle(reactor, SW_FD_CLOSE, swReactorProcess_onClose);
     //pipe
-    reactor->setHandle(reactor, SW_FD_PIPE | SW_EVENT_WRITE, swReactor_onWrite);
+    reactor->setHandle(reactor, SW_FD_WRITE, swReactor_onWrite);
     reactor->setHandle(reactor, SW_FD_PIPE | SW_EVENT_READ, swReactorProcess_onPipeRead);
 
     swServer_store_listen_socket(serv);
