@@ -199,6 +199,10 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_event_defer, 0, 0, 1)
     ZEND_ARG_INFO(0, callback)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_event_cycle, 0, 0, 1)
+    ZEND_ARG_INFO(0, callback)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_event_del, 0, 0, 1)
     ZEND_ARG_INFO(0, fd)
 ZEND_END_ARG_INFO()
@@ -275,6 +279,7 @@ ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_strerror, 0, 0, 1)
     ZEND_ARG_INFO(0, errno)
+    ZEND_ARG_INFO(0, error_type)
 ZEND_END_ARG_INFO()
 
 #ifdef HAVE_PCRE
@@ -315,6 +320,7 @@ const zend_function_entry swoole_functions[] =
     PHP_FE(swoole_event_wait, arginfo_swoole_void)
     PHP_FE(swoole_event_write, arginfo_swoole_event_write)
     PHP_FE(swoole_event_defer, arginfo_swoole_event_defer)
+    PHP_FE(swoole_event_cycle, arginfo_swoole_event_cycle)
     /*------swoole_timer-----*/
     PHP_FE(swoole_timer_after, arginfo_swoole_timer_after)
     PHP_FE(swoole_timer_tick, arginfo_swoole_timer_tick)
@@ -397,6 +403,7 @@ static const zend_function_entry swoole_connection_iterator_methods[] =
     PHP_ME(swoole_connection_iterator, key,         arginfo_swoole_void, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_connection_iterator, valid,       arginfo_swoole_void, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_connection_iterator, count,       arginfo_swoole_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_connection_iterator, __destruct,  arginfo_swoole_void, ZEND_ACC_PUBLIC | ZEND_ACC_DTOR)
     PHP_ME(swoole_connection_iterator, offsetExists,    arginfo_swoole_connection_iterator_offsetExists, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_connection_iterator, offsetGet,       arginfo_swoole_connection_iterator_offsetGet, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_connection_iterator, offsetSet,       arginfo_swoole_connection_iterator_offsetSet, ZEND_ACC_PUBLIC)
@@ -423,6 +430,7 @@ static const zend_function_entry swoole_event_methods[] =
     ZEND_FENTRY(write, ZEND_FN(swoole_event_write), arginfo_swoole_event_write, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     ZEND_FENTRY(wait, ZEND_FN(swoole_event_wait), arginfo_swoole_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     ZEND_FENTRY(defer, ZEND_FN(swoole_event_defer), arginfo_swoole_event_defer, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    ZEND_FENTRY(cycle, ZEND_FN(swoole_event_cycle), arginfo_swoole_event_cycle, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_FE_END
 };
 
@@ -798,6 +806,56 @@ PHP_MINIT_FUNCTION(swoole)
 
     REGISTER_STRINGL_CONSTANT("SWOOLE_VERSION", PHP_SWOOLE_VERSION, sizeof(PHP_SWOOLE_VERSION) - 1, CONST_CS | CONST_PERSISTENT);
 
+    SWOOLE_DEFINE(ERROR_MALLOC_FAIL);
+    SWOOLE_DEFINE(ERROR_SYSTEM_CALL_FAIL);
+    SWOOLE_DEFINE(ERROR_PHP_FATAL_ERROR);
+    SWOOLE_DEFINE(ERROR_NAME_TOO_LONG);
+    SWOOLE_DEFINE(ERROR_INVALID_PARAMS);
+    SWOOLE_DEFINE(ERROR_FILE_NOT_EXIST);
+    SWOOLE_DEFINE(ERROR_FILE_TOO_LARGE);
+    SWOOLE_DEFINE(ERROR_FILE_EMPTY);
+    SWOOLE_DEFINE(ERROR_DNSLOOKUP_DUPLICATE_REQUEST);
+    SWOOLE_DEFINE(ERROR_DNSLOOKUP_RESOLVE_FAILED);
+    SWOOLE_DEFINE(ERROR_SESSION_CLOSED_BY_SERVER);
+    SWOOLE_DEFINE(ERROR_SESSION_CLOSED_BY_CLIENT);
+    SWOOLE_DEFINE(ERROR_SESSION_CLOSING);
+    SWOOLE_DEFINE(ERROR_SESSION_CLOSED);
+    SWOOLE_DEFINE(ERROR_SESSION_NOT_EXIST);
+    SWOOLE_DEFINE(ERROR_SESSION_INVALID_ID);
+    SWOOLE_DEFINE(ERROR_SESSION_DISCARD_TIMEOUT_DATA);
+    SWOOLE_DEFINE(ERROR_OUTPUT_BUFFER_OVERFLOW);
+    SWOOLE_DEFINE(ERROR_SSL_NOT_READY);
+    SWOOLE_DEFINE(ERROR_SSL_CANNOT_USE_SENFILE);
+    SWOOLE_DEFINE(ERROR_SSL_EMPTY_PEER_CERTIFICATE);
+    SWOOLE_DEFINE(ERROR_SSL_VEFIRY_FAILED);
+    SWOOLE_DEFINE(ERROR_SSL_BAD_CLIENT);
+    SWOOLE_DEFINE(ERROR_SSL_BAD_PROTOCOL);
+    SWOOLE_DEFINE(ERROR_PACKAGE_LENGTH_TOO_LARGE);
+    SWOOLE_DEFINE(ERROR_DATA_LENGTH_TOO_LARGE);
+    SWOOLE_DEFINE(ERROR_TASK_PACKAGE_TOO_BIG);
+    SWOOLE_DEFINE(ERROR_TASK_DISPATCH_FAIL);
+    SWOOLE_DEFINE(ERROR_HTTP2_STREAM_ID_TOO_BIG);
+    SWOOLE_DEFINE(ERROR_HTTP2_STREAM_NO_HEADER);
+    SWOOLE_DEFINE(ERROR_SOCKS5_UNSUPPORT_VERSION);
+    SWOOLE_DEFINE(ERROR_SOCKS5_UNSUPPORT_METHOD);
+    SWOOLE_DEFINE(ERROR_SOCKS5_AUTH_FAILED);
+    SWOOLE_DEFINE(ERROR_SOCKS5_SERVER_ERROR);
+    SWOOLE_DEFINE(ERROR_HTTP_PROXY_HANDSHAKE_ERROR);
+    SWOOLE_DEFINE(ERROR_HTTP_INVALID_PROTOCOL);
+    SWOOLE_DEFINE(ERROR_WEBSOCKET_BAD_CLIENT);
+    SWOOLE_DEFINE(ERROR_WEBSOCKET_BAD_OPCODE);
+    SWOOLE_DEFINE(ERROR_WEBSOCKET_UNCONNECTED);
+    SWOOLE_DEFINE(ERROR_WEBSOCKET_HANDSHAKE_FAILED);
+    SWOOLE_DEFINE(ERROR_SERVER_MUST_CREATED_BEFORE_CLIENT);
+    SWOOLE_DEFINE(ERROR_SERVER_TOO_MANY_SOCKET);
+    SWOOLE_DEFINE(ERROR_SERVER_WORKER_TERMINATED);
+    SWOOLE_DEFINE(ERROR_SERVER_INVALID_LISTEN_PORT);
+    SWOOLE_DEFINE(ERROR_SERVER_TOO_MANY_LISTEN_PORT);
+    SWOOLE_DEFINE(ERROR_SERVER_PIPE_BUFFER_FULL);
+    SWOOLE_DEFINE(ERROR_SERVER_NO_IDLE_WORKER);
+    SWOOLE_DEFINE(ERROR_SERVER_ONLY_START_ONE);
+    SWOOLE_DEFINE(ERROR_SERVER_WORKER_EXIT_TIMEOUT);
+
     SWOOLE_INIT_CLASS_ENTRY(swoole_server_ce, "swoole_server", "Swoole\\Server", swoole_server_methods);
     swoole_server_class_entry_ptr = zend_register_internal_class(&swoole_server_ce TSRMLS_CC);
     SWOOLE_CLASS_ALIAS(swoole_server, "Swoole\\Server");
@@ -1112,12 +1170,24 @@ PHP_FUNCTION(swoole_strerror)
 {
     long swoole_errno = 0;
     char error_msg[256] = {0};
+    long error_type = 0;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &swoole_errno) == FAILURE)
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l|l", &swoole_errno, &error_type) == FAILURE)
     {
         return;
     }
-    snprintf(error_msg, sizeof(error_msg) - 1, "%s", strerror(swoole_errno));
+    if (error_type == 1)
+    {
+        snprintf(error_msg, sizeof(error_msg) - 1, "%s", gai_strerror(swoole_errno));
+    }
+    else if (error_type == 2)
+    {
+        snprintf(error_msg, sizeof(error_msg) - 1, "%s", hstrerror(swoole_errno));
+    }
+    else
+    {
+        snprintf(error_msg, sizeof(error_msg) - 1, "%s", strerror(swoole_errno));
+    }
     SW_RETURN_STRING(error_msg, 1);
 }
 
