@@ -85,6 +85,7 @@ static PHP_METHOD(swoole_client, isConnected);
 static PHP_METHOD(swoole_client, getsockname);
 static PHP_METHOD(swoole_client, getpeername);
 static PHP_METHOD(swoole_client, close);
+static PHP_METHOD(swoole_client, shutdown);
 static PHP_METHOD(swoole_client, on);
 
 #ifdef SWOOLE_SOCKETS_SUPPORT
@@ -100,9 +101,6 @@ static void client_onClose(swClient *cli);
 static void client_onError(swClient *cli);
 static void client_onBufferFull(swClient *cli);
 static void client_onBufferEmpty(swClient *cli);
-#ifdef SW_USE_OPENSSL
-static void client_check_ssl_setting(swClient *cli, zval *zset TSRMLS_DC);
-#endif
 
 static sw_inline void client_execute_callback(zval *zobject, enum php_swoole_client_callback_type type)
 {
@@ -258,6 +256,10 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_client_close, 0, 0, 0)
     ZEND_ARG_INFO(0, force)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_client_shutdown, 0, 0, 1)
+    ZEND_ARG_INFO(0, how)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_client_on, 0, 0, 2)
     ZEND_ARG_INFO(0, event_name)
     ZEND_ARG_INFO(0, callback)
@@ -284,6 +286,7 @@ static const zend_function_entry swoole_client_methods[] =
     PHP_ME(swoole_client, wakeup, arginfo_swoole_client_void, ZEND_ACC_PUBLIC)
     PHP_MALIAS(swoole_client, pause, sleep, arginfo_swoole_client_void, ZEND_ACC_PUBLIC)
     PHP_MALIAS(swoole_client, resume, wakeup, arginfo_swoole_client_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_client, shutdown, arginfo_swoole_client_shutdown, ZEND_ACC_PUBLIC)
 #ifdef SW_USE_OPENSSL
     PHP_ME(swoole_client, enableSSL, arginfo_swoole_client_enableSSL, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_client, getPeerCert, arginfo_swoole_client_void, ZEND_ACC_PUBLIC)
@@ -337,12 +340,10 @@ void swoole_client_init(int module_number TSRMLS_DC)
     zend_declare_class_constant_long(swoole_client_class_entry_ptr, ZEND_STRL("MSG_PEEK"), MSG_PEEK TSRMLS_CC);
     zend_declare_class_constant_long(swoole_client_class_entry_ptr, ZEND_STRL("MSG_DONTWAIT"), MSG_DONTWAIT TSRMLS_CC);
     zend_declare_class_constant_long(swoole_client_class_entry_ptr, ZEND_STRL("MSG_WAITALL"), MSG_WAITALL TSRMLS_CC);
-}
 
-int php_swoole_client_onPackage(swConnection *conn, char *data, uint32_t length)
-{
-    client_onReceive(conn->object, data, length);
-    return SW_OK;
+    zend_declare_class_constant_long(swoole_client_class_entry_ptr, ZEND_STRL("SHUT_RDWR"), SHUT_RDWR TSRMLS_CC);
+    zend_declare_class_constant_long(swoole_client_class_entry_ptr, ZEND_STRL("SHUT_RD"), SHUT_RD TSRMLS_CC);
+    zend_declare_class_constant_long(swoole_client_class_entry_ptr, ZEND_STRL("SHUT_WR"), SHUT_WR TSRMLS_CC);
 }
 
 static void client_onReceive(swClient *cli, char *data, uint32_t length)
@@ -455,7 +456,7 @@ static void client_onBufferEmpty(swClient *cli)
 }
 
 #ifdef SW_USE_OPENSSL
-static void client_check_ssl_setting(swClient *cli, zval *zset TSRMLS_DC)
+void php_swoole_client_check_ssl_setting(swClient *cli, zval *zset TSRMLS_DC)
 {
     HashTable *vht = Z_ARRVAL_P(zset);
     zval *v;
@@ -496,12 +497,37 @@ static void client_check_ssl_setting(swClient *cli, zval *zset TSRMLS_DC)
         cli->ssl_option.passphrase = sw_strdup(Z_STRVAL_P(v));
     }
 #ifdef SSL_CTRL_SET_TLSEXT_HOSTNAME
-    if (php_swoole_array_get_value(vht, "tls_host_name", v))
+    if (php_swoole_array_get_value(vht, "ssl_host_name", v))
     {
         convert_to_string(v);
         cli->ssl_option.tls_host_name = sw_strdup(Z_STRVAL_P(v));
     }
 #endif
+    if (php_swoole_array_get_value(vht, "ssl_verify_peer", v))
+    {
+        convert_to_boolean(v);
+        cli->ssl_option.verify_peer = Z_BVAL_P(v);
+    }
+    if (php_swoole_array_get_value(vht, "ssl_allow_self_signed", v))
+    {
+        convert_to_boolean(v);
+        cli->ssl_option.allow_self_signed = Z_BVAL_P(v);
+    }
+    if (php_swoole_array_get_value(vht, "ssl_cafile", v))
+    {
+        convert_to_string(v);
+        cli->ssl_option.cafile = sw_strdup(Z_STRVAL_P(v));
+    }
+    if (php_swoole_array_get_value(vht, "ssl_capath", v))
+    {
+        convert_to_string(v);
+        cli->ssl_option.capath = sw_strdup(Z_STRVAL_P(v));
+    }
+    if (php_swoole_array_get_value(vht, "ssl_verify_depth", v))
+    {
+        convert_to_long(v);
+        cli->ssl_option.verify_depth = (int) Z_LVAL_P(v);
+    }
     if (cli->ssl_option.cert_file && !cli->ssl_option.key_file)
     {
         swoole_php_fatal_error(E_ERROR, "ssl require key file.");
@@ -573,7 +599,6 @@ void php_swoole_client_check_setting(swClient *cli, zval *zset TSRMLS_DC)
         }
         bzero(cli->protocol.package_eof, SW_DATA_EOF_MAXLEN);
         memcpy(cli->protocol.package_eof, Z_STRVAL_P(v), Z_STRLEN_P(v));
-        cli->protocol.onPackage = php_swoole_client_onPackage;
     }
     //open mqtt protocol
     if (php_swoole_array_get_value(vht, "open_mqtt_protocol", v))
@@ -581,7 +606,6 @@ void php_swoole_client_check_setting(swClient *cli, zval *zset TSRMLS_DC)
         convert_to_boolean(v);
         cli->open_length_check = Z_BVAL_P(v);
         cli->protocol.get_package_length = swMqtt_get_package_length;
-        cli->protocol.onPackage = php_swoole_client_onPackage;
     }
     //open length check
     if (php_swoole_array_get_value(vht, "open_length_check", v))
@@ -589,7 +613,6 @@ void php_swoole_client_check_setting(swClient *cli, zval *zset TSRMLS_DC)
         convert_to_boolean(v);
         cli->open_length_check = Z_BVAL_P(v);
         cli->protocol.get_package_length = swProtocol_get_package_length;
-        cli->protocol.onPackage = php_swoole_client_onPackage;
     }
     //package length size
     if (php_swoole_array_get_value(vht, "package_length_type", v))
@@ -726,7 +749,7 @@ void php_swoole_client_check_setting(swClient *cli, zval *zset TSRMLS_DC)
         convert_to_string(v);
         cli->socks5_proxy = emalloc(sizeof(swSocks5));
         bzero(cli->socks5_proxy, sizeof(swSocks5));
-        cli->socks5_proxy->host = sw_strdup(Z_STRVAL_P(v));
+        cli->socks5_proxy->host = estrdup(Z_STRVAL_P(v));
         cli->socks5_proxy->dns_tunnel = 1;
 
         if (php_swoole_array_get_value(vht, "socks5_port", v))
@@ -742,14 +765,14 @@ void php_swoole_client_check_setting(swClient *cli, zval *zset TSRMLS_DC)
         if (php_swoole_array_get_value(vht, "socks5_username", v))
         {
             convert_to_string(v);
-            cli->socks5_proxy->username = Z_STRVAL_P(v);
+            cli->socks5_proxy->username = estrdup(Z_STRVAL_P(v));
             cli->socks5_proxy->l_username = Z_STRLEN_P(v);
             cli->socks5_proxy->method = 0x02;
         }
         if (php_swoole_array_get_value(vht, "socks5_password", v))
         {
             convert_to_string(v);
-            cli->socks5_proxy->password = Z_STRVAL_P(v);
+            cli->socks5_proxy->password = estrdup(Z_STRVAL_P(v));
             cli->socks5_proxy->l_password = Z_STRLEN_P(v);
         }
     }
@@ -760,19 +783,41 @@ void php_swoole_client_check_setting(swClient *cli, zval *zset TSRMLS_DC)
         if (php_swoole_array_get_value(vht, "http_proxy_port", v))
         {
             convert_to_long(v);
-            cli->http_proxy = ecalloc(1,sizeof(struct _http_proxy));
-            cli->http_proxy->proxy_host = sw_strdup(host);
+            cli->http_proxy = ecalloc(1, sizeof(struct _http_proxy));
+            cli->http_proxy->proxy_host = estrdup(host);
             cli->http_proxy->proxy_port = Z_LVAL_P(v);
         }
         else
         {
-            swSysError("http_proxy_port can not be null");
+            swoole_php_fatal_error(E_WARNING, "http_proxy_port can not be null.");
+        }
+    }
+    if (php_swoole_array_get_value(vht, "http_proxy_user", v) && cli->http_proxy)
+    {
+        convert_to_string(v);
+        char *user = Z_STRVAL_P(v);
+        zval *v2;
+        if (php_swoole_array_get_value(vht, "http_proxy_password", v2))
+        {
+            convert_to_string(v);
+            if (Z_STRLEN_P(v) + Z_STRLEN_P(v2) >= 128 - 1)
+            {
+                swoole_php_fatal_error(E_WARNING, "http_proxy user and password is too long.");
+            }
+            cli->http_proxy->l_user = Z_STRLEN_P(v);
+            cli->http_proxy->l_password = Z_STRLEN_P(v2);
+            cli->http_proxy->user = estrdup(user);
+            cli->http_proxy->password = estrdup(Z_STRVAL_P(v2));
+        }
+        else
+        {
+            swoole_php_fatal_error(E_WARNING, "http_proxy_password can not be null.");
         }
     }
 #ifdef SW_USE_OPENSSL
     if (cli->open_ssl)
     {
-        client_check_ssl_setting(cli, zset TSRMLS_CC);
+        php_swoole_client_check_ssl_setting(cli, zset TSRMLS_CC);
     }
 #endif
 }
@@ -894,11 +939,29 @@ void php_swoole_client_free(zval *zobject, swClient *cli TSRMLS_DC)
     //socks5 proxy config
     if (cli->socks5_proxy)
     {
+        efree(cli->socks5_proxy->host);
+        if (cli->socks5_proxy->username)
+        {
+            efree(cli->socks5_proxy->username);
+        }
+        if (cli->socks5_proxy->password)
+        {
+            efree(cli->socks5_proxy->password);
+        }
         efree(cli->socks5_proxy);
     }
     //http proxy config
     if (cli->http_proxy)
     {
+        efree(cli->http_proxy->proxy_host);
+        if (cli->http_proxy->user)
+        {
+            efree(cli->http_proxy->user);
+        }
+        if (cli->http_proxy->password)
+        {
+            efree(cli->http_proxy->password);
+        }
         efree(cli->http_proxy);
     }
     if (cli->protocol.private_data)
@@ -1119,6 +1182,10 @@ static PHP_METHOD(swoole_client, set)
     {
         return;
     }
+    if (Z_TYPE_P(zset) != IS_ARRAY)
+    {
+        RETURN_FALSE;
+    }
 
     zval *zsetting = php_swoole_read_init_property(swoole_client_class_entry_ptr, getThis(), ZEND_STRL("setting") TSRMLS_CC);
     sw_php_array_merge(Z_ARRVAL_P(zsetting), Z_ARRVAL_P(zset));
@@ -1953,7 +2020,7 @@ static PHP_METHOD(swoole_client, enableSSL)
     zval *zset = sw_zend_read_property(swoole_client_class_entry_ptr, getThis(), ZEND_STRL("setting"), 1 TSRMLS_CC);
     if (zset && !ZVAL_IS_NULL(zset))
     {
-        client_check_ssl_setting(cli, zset TSRMLS_CC);
+        php_swoole_client_check_ssl_setting(cli, zset TSRMLS_CC);
     }
     if (swClient_enable_ssl_encrypt(cli) < 0)
     {
@@ -2053,7 +2120,7 @@ static PHP_METHOD(swoole_client, verifyPeerCert)
     {
         return;
     }
-    SW_CHECK_RETURN(swSSL_verify(cli->socket, allow_self_signed));
+    SW_CHECK_RETURN(swClient_ssl_verify(cli, allow_self_signed));
 }
 #endif
 
@@ -2095,6 +2162,21 @@ static PHP_METHOD(swoole_client, pipe)
     SW_CHECK_RETURN(cli->pipe(cli, fd, flags));
 }
 
+static PHP_METHOD(swoole_client, shutdown)
+{
+    swClient *cli = client_get_ptr(getThis() TSRMLS_CC);
+    if (!cli)
+    {
+        RETURN_FALSE;
+    }
+    long __how;
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &__how) == FAILURE)
+    {
+        return;
+    }
+    SW_CHECK_RETURN(swClient_shutdown(cli, __how));
+}
+
 PHP_FUNCTION(swoole_client_select)
 {
     zval *r_array, *w_array, *e_array;
