@@ -24,6 +24,7 @@
 #endif
 
 #define SW_MAX_FDTYPE              32   //32 kinds of event
+#define SW_MAX_HOOK_TYPE           32
 #define SW_ERROR_MSG_SIZE          512
 #define SW_MAX_WORKER_GROUP        2
 #define SW_MAX_FILE_CONTENT        (64*1024*1024) //for swoole_file_get_contents
@@ -105,8 +106,6 @@
 #define SW_HEARTBEAT_PING_LEN      8
 #define SW_HEARTBEAT_PONG_LEN      8
 
-#define SW_MAINREACTOR_USE_UNSOCK  1    //主线程使用unsock
-#define SW_REACTOR_WRITER_TIMEO    3    //writer线程的reactor
 #define SW_TASKWAIT_TIMEOUT        0.5
 
 #ifdef HAVE_EVENTFD
