@@ -419,16 +419,19 @@ static PHP_METHOD(swoole_redis, close)
 static PHP_METHOD(swoole_redis, __destruct)
 {
     swRedisClient *redis = swoole_get_object(getThis());
-    if (redis && redis->context && redis->state != SWOOLE_REDIS_STATE_CLOSED)
+    if (redis)
     {
-        redisAsyncDisconnect(redis->context);
+        if (redis->context && redis->state != SWOOLE_REDIS_STATE_CLOSED)
+        {
+            redisAsyncDisconnect(redis->context);
+        }
+        if (redis->password)
+        {
+            efree(redis->password);
+        }
+        efree(redis);
+        swoole_set_object(getThis(), NULL);
     }
-    swoole_set_object(getThis(), NULL);
-    if (redis->password)
-    {
-        efree(redis->password);
-    }
-    efree(redis);
 }
 
 static PHP_METHOD(swoole_redis, __call)
@@ -887,7 +890,7 @@ void swoole_redis_onClose(const redisAsyncContext *c, int status)
     redis->state = SWOOLE_REDIS_STATE_CLOSED;
 
 #if PHP_MAJOR_VERSION < 7
-        TSRMLS_FETCH_FROM_CTX(sw_thread_ctx ? sw_thread_ctx : NULL);
+    TSRMLS_FETCH_FROM_CTX(sw_thread_ctx ? sw_thread_ctx : NULL);
 #endif
 
     zval *zcallback = sw_zend_read_property(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("onClose"), 1 TSRMLS_CC);
